
#include "pg.h"
#include "font.h"
#include <pspdisplay.h>
#include <pspiofilemgr.h>
#include <string.h>
#include <psptypes.h>



#define RGB5( r, g, b )	( ( ( b ) << 10 ) + ( ( g ) << 5 ) + ( ( r ) << 0 ) + 0x8000 )
#define R5( c )			( ( ( c ) >> 0 ) & 31 )
#define G5( c )			( ( ( c ) >> 5 ) & 31 )
#define B5( c )			( ( ( c ) >> 10 ) & 31 )



//variables
char *pg_vramtop = (char *)0x04000000;
long pg_screenmode;
long pg_showframe;
long pg_drawframe;


	


	




void pgWaitVn(unsigned long count)
{
	for (; count>0; --count) {
		sceDisplayWaitVblankStart();
	}
}


void pgWaitV()
{
	sceDisplayWaitVblankStart();
}


char *pgGetVramAddr(unsigned long x,unsigned long y)
{
	return pg_vramtop+(pg_drawframe?FRAMESIZE:0)+x*PIXELSIZE*2+y*LINESIZE*2+0x40000000;
}


void pgInit()
{
	sceDisplaySetMode(0,SCREEN_WIDTH,SCREEN_HEIGHT);
	pgScreenFrame(0,0);
}



void pgPrint(unsigned long x,unsigned long y,unsigned long color,const char *str)
{
	while (*str!=0 && x<CMAX_X && y<CMAX_Y) {
		pgPutChar(x*8,y*8,color,0,*str,1,0,1);
		str++;
		x++;
		if (x>=CMAX_X) {
			x=0;
			y++;
		}
	}
}


void pgPrint2(unsigned long x,unsigned long y,unsigned long color,const char *str)
{
	while (*str!=0 && x<CMAX2_X && y<CMAX2_Y) {
		pgPutChar(x*16,y*16,color,0,*str,1,0,2);
		str++;
		x++;
		if (x>=CMAX2_X) {
			x=0;
			y++;
		}
	}
}


void pgPrint4(unsigned long x,unsigned long y,unsigned long color,const char *str)
{
	while (*str!=0 && x<CMAX4_X && y<CMAX4_Y) {
		pgPutChar(x*32,y*32,color,0,*str,1,0,4);
		str++;
		x++;
		if (x>=CMAX4_X) {
			x=0;
			y++;
		}
	}
}


void pgFillvram(unsigned long color)
{
	char *vptr0;		//pointer to vram
	unsigned long i;

	vptr0=pgGetVramAddr(0,0);
	for (i=0; i<FRAMESIZE/2; i++) {
		*(unsigned short *)vptr0=color;
		vptr0+=PIXELSIZE*2;
	}
}

void pgBitBlt(unsigned long x,unsigned long y,unsigned long w,unsigned long h,unsigned long mag,const unsigned short *d)
{
	char *vptr0;		//pointer to vram
	char *vptr;		//pointer to vram
	unsigned long xx,yy,mx,my;
	const unsigned short *dd;
	
	vptr0=pgGetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			dd=d;
			for (xx=0; xx<w; xx++) {
				for (mx=0; mx<mag; mx++) {
					*(unsigned short *)vptr=*dd;
					vptr+=PIXELSIZE*2;
				}
				dd++;
			}
			vptr0+=LINESIZE*2;
		}
		d+=w;
	}
	
}


void pgPutChar(unsigned long x,unsigned long y,unsigned long color,unsigned long bgcolor,const unsigned char ch,char drawfg,char drawbg,char mag)
{
	char *vptr0;		//pointer to vram
	char *vptr;		//pointer to vram
	const unsigned char *cfont;		//pointer to font
	unsigned long cx,cy;
	unsigned long b;
	char mx,my;

	cfont=font+ch*8;
	vptr0=pgGetVramAddr(x,y);
	for (cy=0; cy<8; cy++) {
		for (my=0; my<mag; my++) {
			vptr=vptr0;
			b=0x80;
			for (cx=0; cx<8; cx++) {
				for (mx=0; mx<mag; mx++) {
					if ((*cfont&b)!=0) {
						if (drawfg) *(unsigned short *)vptr=color;
					} else {
						if (drawbg) *(unsigned short *)vptr=bgcolor;
					}
					vptr+=PIXELSIZE*2;
				}
				b=b>>1;
			}
			vptr0+=LINESIZE*2;
		}
		cfont++;
	}
}


void pgScreenFrame(long mode,long frame)
{
	pg_screenmode=mode;
	frame=(frame?1:0);
	pg_showframe=frame;
	if (mode==0) {
		//screen off
		pg_drawframe=frame;
		sceDisplaySetFrameBuf(0,0,0,1);
	} else if (mode==1) {
		//show/draw same
		pg_drawframe=frame;
		sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	} else if (mode==2) {
		//show/draw different
		pg_drawframe=(frame?0:1);
		sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,1);
	}
}


void pgScreenFlip()
{
	pg_showframe=(pg_showframe?0:1);
	pg_drawframe=(pg_drawframe?0:1);
	sceDisplaySetFrameBuf(pg_vramtop+(pg_showframe?FRAMESIZE:0),LINESIZE,PIXELSIZE,0);
}


void pgScreenFlipV()
{
	pgWaitV();
	pgScreenFlip();
}







unsigned long pgc_csr_x[2], pgc_csr_y[2];
unsigned long pgc_fgcolor[2], pgc_bgcolor[2];
char pgc_fgdraw[2], pgc_bgdraw[2];
char pgc_mag[2];





void pgMoverect(unsigned long x, unsigned long y, long w, long h, unsigned long tx, unsigned long ty)
{
	char *vptr_src, *vptr_dest;
	long yy,dy,ofs,end;

	if (h>=0) {
		dy=LINESIZE*2;
	} else {
		dy=-LINESIZE*2;
		h=-h;
	}

	vptr_src=pgGetVramAddr(x,y);
	vptr_dest=pgGetVramAddr(tx,ty);
	
	if (w>=0) {
		for (yy=h; yy>0; --yy) {
			end=PIXELSIZE*2*w;
			for (ofs=0; ofs!=end; ofs+=PIXELSIZE*2) {
				*(unsigned short *)(vptr_dest+ofs)=*(unsigned short *)(vptr_src+ofs);
			}
			vptr_src+=dy;
			vptr_dest+=dy;
		}
	} else {
		for (yy=h; yy>0; --yy) {
			end=PIXELSIZE*2*w;
			for (ofs=0; ofs!=end; ofs-=PIXELSIZE*2) {
				*(unsigned short *)(vptr_dest+ofs)=*(unsigned short *)(vptr_src+ofs);
			}
			vptr_src+=dy;
			vptr_dest+=dy;
		}
	}
}




void pgFillrect(unsigned long x, unsigned long y, long w, long h, unsigned long color)
{
	char *vptr0;
	char *vptr;
	long xx,yy;

	if (w<0) w=-w;
	if (h<0) h=-h;
	vptr0=pgGetVramAddr(x,y);
	for (yy=0; yy<h; yy++) {
		vptr=vptr0;
		for (xx=0; xx<w; xx++) {
			*(unsigned short *)vptr=color;
			vptr+=PIXELSIZE*2;
		}
		vptr0+=LINESIZE*2;
	}
}





/******************************************************************************/


void pgcLocate(unsigned long x, unsigned long y)
{
	if (x>=CMAX_X) x=0;
	if (y>=CMAX_Y) y=0;
	pgc_csr_x[pg_drawframe?1:0]=x;
	pgc_csr_y[pg_drawframe?1:0]=y;
}


void pgcColor(unsigned long fg, unsigned long bg)
{
	pgc_fgcolor[pg_drawframe?1:0]=fg;
	pgc_bgcolor[pg_drawframe?1:0]=bg;
}


void pgcDraw(char drawfg, char drawbg)
{
	pgc_fgdraw[pg_drawframe?1:0]=drawfg;
	pgc_bgdraw[pg_drawframe?1:0]=drawbg;
}


void pgcSetmag(char mag)
{
	pgc_mag[pg_drawframe?1:0]=mag;
}


void pgcPutchar(const char ch)
{
	if (ch==0x0d) {
		pgc_csr_x[pg_drawframe]=0;
		return;
	}
	if (ch==0x0a) {
		if ((++pgc_csr_y[pg_drawframe])>=CMAX_Y) {
			pgc_csr_y[pg_drawframe]=CMAX_Y-1;
			pgMoverect(0,8,SCREEN_WIDTH,SCREEN_HEIGHT-8,0,0);
		}
		return;
	}
	pgcPutchar_nocontrol(ch);
}


void pgcPutchar_nocontrol(const char ch)
{
	pgPutChar(pgc_csr_x[pg_drawframe]*8, pgc_csr_y[pg_drawframe]*8, pgc_fgcolor[pg_drawframe], pgc_bgcolor[pg_drawframe], ch, pgc_fgdraw[pg_drawframe], pgc_bgdraw[pg_drawframe], pgc_mag[pg_drawframe]);
	pgc_csr_x[pg_drawframe]+=pgc_mag[pg_drawframe];
	if (pgc_csr_x[pg_drawframe]>CMAX_X-pgc_mag[pg_drawframe]) {
		pgc_csr_x[pg_drawframe]=0;
		pgc_csr_y[pg_drawframe]+=pgc_mag[pg_drawframe];
		if (pgc_csr_y[pg_drawframe]>CMAX_Y-pgc_mag[pg_drawframe]) {
			pgc_csr_y[pg_drawframe]=CMAX_Y-pgc_mag[pg_drawframe];
			pgMoverect(0,pgc_mag[pg_drawframe]*8,SCREEN_WIDTH,SCREEN_HEIGHT-pgc_mag[pg_drawframe]*8,0,0);
		}
	}
}


void pgcPuts(const char *str)
{
	for (; *str!=0; ++str) pgcPutchar(*str);
}


void pgcPutsn(const char *str,unsigned long max)
{
	for (; *str!=0 && max>0; ++str, --max) pgcPutchar(*str);
}





void pgcCls()
{
	pgFillvram(pgc_bgcolor[pg_drawframe]);
	pgcLocate(0,0);
}


void pgcClearlines(unsigned long st, unsigned long ed)
{
	if (st>=CMAX_Y) return;
	if (ed>=CMAX_Y) ed=CMAX_Y-1;
	if (st>ed) return;
	pgFillrect(0,st*8,SCREEN_WIDTH,(ed-st)*8+8,pgc_bgcolor[pg_drawframe]);
}


void pgcCleareol()
{
	pgFillrect(pgc_csr_x[pg_drawframe]*8,pgc_csr_y[pg_drawframe]*8,SCREEN_WIDTH-pgc_csr_x[pg_drawframe]*8,8,pgc_bgcolor[pg_drawframe]);
}



/******************************************************************************/





/******************************************************************************/






void pgErrorHalt(const char *str)
{
	pgScreenFrame(1,0);
	pgcLocate(0,0);
	pgcColor(0xffff,0x0000);
	pgcDraw(1,1);
	pgcSetmag(1);
	pgcPuts(str);
	while (1) { pgWaitV(); }
}





